#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "odicon.h"
#include "lseparable.h"
#include "laminar.h"


static struct lseparable_st *lseparable_inst;
static double ffuncL(int dim, double x[]) { return lseparable(dim, x, lseparable_inst); }
static double ifuncL(int dim, int x[]) { return intint_func(dim, ffuncL, x); }

static int v[1000];
static double ffuncM(int dim, double x[]) { return flaminar2(dim, x, v); }
static double ifuncM(int dim, int x[]) { return intint_func(dim, ffuncM, x); }

static void read_Lparam(int *init, struct lseparable_st *p) {
    if (!imatrix_scan(p->dim, p->aa)) return;
    if (!imatrix_scan(p->dim, p->bb)) return;
    if (!ivector_scan(p->dim, p->a)) return;
    if (!ivector_scan(p->dim, p->b)) return;
    if (!ivector_scan(p->dim, p->c)) return;
    if (!ivector_scan(p->dim, init)) return;
}

static void read_Mparam(const int dim, int *init, int *v) {
    int i, j, k, index, a[100], b[100];

    if (!ivector_scan(dim * 2 - 1, a)) return;
    if (!ivector_scan(dim * 2 - 1, b)) return;
    if (!ivector_scan(dim, init)) return;

    index = 0;
    for (i=1; i/2<dim; i*=2) {
	for (j=0; j+i/2<dim; j+=i) {
            int max = j+i;  if (max > dim) max = dim;
            *v++ = a[index];
            *v++ = b[index];
            *v++ = 0;
            index++;
            for (k=j; k<max; k++) *v++ = k;
            *v++ = -1; /* end mark */
	}
    }
    *v++ = 0;
    *v++ = 0;
    *v++ = 0; /* end mark */
}

void demo_Lseparable(const int dim) {
    int init[1000];

    lseparable_inst = lseparable_make(dim);
    read_Lparam(init, lseparable_inst);
    /* lseparable_output(lseparable_inst); */

    lgconv_minimize(dim, ifuncL, init, 0);
}

void demo_Mseparable(const int dim) {
    int init[1000];

    read_Mparam(dim, init, v);

    mgconv_minimize(dim, ifuncM, init);
}
