#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "odicon.h"
#include "laminar.h"

double flaminartt(const int dim, double x[], struct laminar_st *p) {
    int i, j;
    double f = 0;

    assert(p->dim == dim);
    for (i=1; i<2*dim; i++) {
        double term = 0;
        for (j=1; j<=dim; j++) {
            if (p->tt[i][j] == 0) break;
            term += x[p->tt[i][j]-1];
        }
        f += p->a[i]*term*term + p->b[i]*term + dim*dim/2;
    }
    return f;
}


double ilaminartt(const int dim, int x[], struct laminar_st *p) {
    int i, j;
    double f = 0;

    assert(p->dim == dim);
    for (i=1; i<2*dim; i++) {
        int term = 0;
        for (j=1; j<=dim; j++) {
            if (p->tt[i][j] == 0) break;
            term += x[p->tt[i][j]-1];
        }
        f += p->a[i]*term*term + p->b[i]*term + dim*dim/2;
    }
    return f;
}


static void make_tree(int dim, int **tt) {
    int i,j,k,a,b,itter,ocardb;
    int *card;

    /*	initialize	*/
    card = xmalloc(sizeof(*card) * 2*dim);
    for (i=1; i<=dim; i++) {
        tt[i][1] = i;
        card[i] = 1;
    }
    for (i=dim+1; i<2*dim; i++) {
        tt[i][1] = 0;
        card[i] = 0;
    }
  
    k = dim;
    for (itter=1; itter<dim; itter++) {
        a = irandom(1, k);
        do { b = irandom(1, k); } while (a == b);
        if (a > b) { int swap=a; a=b; b=swap; }
      
        for (j=1; j<=card[b]; j++) {
            tt[a][card[a]+j] = tt[b][j];
        }
      
        ocardb = card[b];
      
        for (i=b; i<=dim+itter-2; i++) {
            for (j=1; j<=dim; j++) {
                tt[i][j] = tt[i+1][j];
                card[i]=card[i+1];
            }
        }

        for (j=1; j<=card[a]; j++) {
            tt[dim+itter-1][j] = tt[a][j];
        }
        for (j=card[a]+1; j<=dim; j++) {
            tt[dim+itter-1][j] = 0;
        }
        card[dim+itter-1]=card[a];

        for (j=1; j<=ocardb; j++) {
            tt[dim+itter][j] = tt[a][card[a]+j];
        }
        card[a]+=ocardb;
        card[dim+itter]=ocardb;

        k--;
    }
    free(card);
}

struct laminar_st *laminertt_make(int dim) {
    int i, j;
    struct laminar_st *p;

    p    = xmalloc(sizeof(p[0]));
    p->dim = dim;
    p->a = xmalloc(sizeof(p->a[0]) * 2*dim);
    p->b = xmalloc(sizeof(p->b[0]) * 2*dim);
    p->c = xmalloc(sizeof(p->c[0]) * 2*dim);
    p->tt = imatrix_new(2*dim, dim+1);
    for (i=0; i<2*dim; i++) for (j=0; j<dim+1; j++) p->tt[i][j] = 0;

    for(i=1; i<2*dim; i++) {
        do { p->a[i] = frandom(0, 1000); } while (p->a[i] == 0);
        p->b[i] = frandom(-1000, 1000);
    }
    make_tree(dim, p->tt);
    return p;
}


void laminar_print(struct laminar_st *p) {
    int i, j;
    const int dim = p->dim;

    printf("S = 1 .. %d;\n", dim);
    printf("T = 1 .. %d;\n", 2*dim-1);

    printf("a=\n");
    for(i=1; i<2*dim; i++) printf("[%d] %g\n", i, p->a[i]);
    printf(";\n");

    printf("b=\n");
    for(i=1; i<2*dim; i++) printf("[%d] %g\n", i, p->b[i]);
    printf(";\n");

    printf("c=\n");
    for(i=1; i<2*dim; i++) printf("[%d] %g\n", i, p->c[i]);
    printf(";\n");

    printf("tt=\n");
    for(i=1; i<2*dim; i++){
        for(j=1; j<=dim; j++){
            if (p->tt[i][j] == 0) break;
            printf("[%d,%d] 1 ", i, p->tt[i][j]);
        }
        printf("\n");
    }
    printf(";\n");
}



/* laminar2 = sum(ax^2 + bx +c) */

double flaminar2(const int dim, double x[], int v[]) {
    int a, b, c;
    double f = 0;
    int i;

    for (i=0; /**/; i++) {
        double term = 0;
        a = v[i++];
        b = v[i++];
        c = v[i++];
        if (a == 0 && b == 0 && c == 0) break;
        while (v[i] >= 0) {
            assert(dim > v[i]);
            term += x[v[i++]];
        }
        f += a*term*term + b*term + c;
    }
    return f;
}

double ilaminar2(const int dim, int x[], int v[]) {
    int a, b, c;
    double f = 0;
    int i;

    for (i=0; /**/; i++) {
        int term = 0;
        a = v[i++];
        b = v[i++];
        c = v[i++];
        if (a == 0 && b == 0 && c == 0) break;
        while (v[i] >= 0) {
            assert(dim > v[i]);
            term += x[v[i++]];
        }
        f += a*term*term + b*term + c;
    }
    return f;
}
